import processing.core.*; 
import processing.xml.*; 

import jm.midi.event.*; 
import jm.music.data.*; 
import jm.music.tools.*; 
import jm.audio.*; 
import jm.audio.synth.*; 
import jm.music.net.*; 
import jm.audio.math.*; 
import jm.util.*; 
import jm.midi.*; 
import jm.music.tools.fuzzy.*; 
import jm.*; 
import jm.constants.*; 
import jm.audio.io.*; 
import jm.music.rt.*; 

import java.applet.*; 
import java.awt.Dimension; 
import java.awt.Frame; 
import java.awt.event.MouseEvent; 
import java.awt.event.KeyEvent; 
import java.awt.event.FocusEvent; 
import java.awt.Image; 
import java.io.*; 
import java.net.*; 
import java.text.*; 
import java.util.*; 
import java.util.zip.*; 
import java.util.regex.*; 

public class GridMusic extends PApplet {

/**
* Nels Oscar
* 2010
* A single instrument midi sequencer built with 
* algomuse, a slightly modified version of jMusic
*/

















int blockW, blockH;
int numRows=60;
int timer=0;
PFont font;
Grid grid;

Muse m;

// GUI declarations
Button play;
Button stop;
Button refresh;
Button sav;

public void setup(){
  size(600,700,P2D);
  background(10,20,40);
  m=new Muse();
  font = loadFont("BookmanOldStyle-Bold-20.vlw");
  play = new Button("Play", 10, 620, 55, 30, font);
  stop = new Button("Stop", 75, 620, 55, 30, font);
  refresh = new Button("Refresh", 140, 620, 90, 30, font);
  sav = new Button("Save", 240, 620, 55, 30, font);
  grid=new Grid(width,height-100,numRows);
  setupGUI();
}

public void draw(){
  checkRollover(mouseX,mouseY);
}

public void setupGUI(){
  play.show();
  stop.show();
  refresh.show();
  sav.show();
}

public void keyPressed(){
  if(key=='r'){
    rand(.01f);
  }
}

public void mousePressed(){
  if(mouseY<=600)
    grid.update(mouseX,mouseY);
  else{
    if(play.contains(mouseX,mouseY)&&timer<=0)
      plaY();
    if(stop.contains(mouseX,mouseY))
      stoP();
    if(refresh.contains(mouseX,mouseY))
      refresH();
    if(sav.contains(mouseX, mouseY))
      savE();
  }
  frameRate(12);
}

public void checkRollover(int x, int y){
  play.live(play.contains(x,y));
  stop.live(stop.contains(x,y));
  refresh.live(refresh.contains(x,y));
  sav.live(sav.contains(x,y));
}

public void plaY(){
  int inst=m.VIOLA;
  Phrase[] phr=grid.parseGrid();
  println("--Retrieved Score Information--");
  for(int i=0; i< phr.length; i++){
    println("--Added Part--");
    Part p=new Part(inst);
    p.addPhrase(phr[i]);
    println("--Added phrase--");
    println(phr[i]);
    m.score.addPart(p);
  }
  inst++;
  println("--Set Score--");
  m.setScore();
  println("PlaY");
  timer=grid.getLongestRow();
  m.play();
}

public void stoP(){
  m.stop();
  println("StoP");
  
}

public void refresH(){
  grid.clear();
  println("RefReSH");
}

public void savE(){
  int inst=m.VIOLA;
  Phrase[] phr=grid.parseGrid();
  println("--Retrieved Score Information--");
  for(int i=0; i< phr.length; i++){
    println("--Added Part--");
    Part p=new Part(inst);
    p.addPhrase(phr[i]);
    println("--Added phrase--");
    println(phr[i]);
    m.score.addPart(p);
  }
  inst++;
  println("--Set Score--");
  m.setScore();
  println("PlaY");
  Write.midi(m.score,sketchPath("test.mid"));
  m.score.empty();
}

// Density must be less than one
public void rand(double density){
  for(int i=0; i<grid.grid.length*density; i++){
    grid.grid[(int)random(0,grid.grid.length-1)].live();
  }
  grid.showBlocks();
}
public class Block{
  
  int bX,bY,bW,bH;
  boolean lit=false;
  int dead=color(0);
  int alive=color(150);
  
  public Block(int x, int y, int w, int h){
    bX=x;
    bY=y;
    bW=w;
    bH=h;
    
  }
  
  public void live(){
    lit=!lit;
  }
  
  public boolean isLit(){
    return lit;
  }
  
  public void show(){
    if(lit){
      fill(alive);
      stroke(dead);
    }else{
      fill(dead);
      stroke(alive);
    }
    rect(bX,bY,bW,bH);
  }
  
  public boolean contains(int x, int y){
    return (bX <= x && x < bX + bW &&
            bY <= y && y < bY + bH   );
  }
  
  public void clear(){
    lit=false;
  }
  
  public String toString(){
    return("--x: "+bX+" y: "+bY+" lit--");
  } 
}
/**
 * @author Nels Oscar
 * 2010
 * GUI elements that handle rollover.
 */
public class Button{
  
  String name;
  
  int w,h,x,y;
  
  PFont font;
  
  boolean mouseOver=false;
  
  int dead=color(0);
  
  int alive=color(150);
  
  
  public Button(String n, int X, int Y, int W, int H,PFont f){
    name=n;
    x=X;
    y=Y;
    w=W;
    h=H;
    font=f;
  }
  
  public void show(){  
    if(mouseOver){
      fill(alive);
      stroke(dead);
    }else{
      fill(dead);
      stroke(alive);
    }
    rect(x,y,w,h);
    if(mouseOver){
      fill(dead);
    }else{
      fill(alive);
    }
    textFont(font);
    text(name,x+5,y+20);
  }
  
  public void live(boolean que){
    mouseOver=que;
    show();
  }
  
  public boolean contains(int X, int Y){
    return (x <= X && X < x + w &&
            y <= Y && Y < y + h   );
  }
}
/**
 * Nels Oscar
 * 2010
 * A block handler to abstract the end user away from handling individual
 * graphics compnents to get at useful data.
 */

public class Grid{
  
  Block[] grid;

  int w,h,numR;
  
  int spaceX;
  int spaceY;
  
  public Grid(int W, int H, int num){
    w=W;
    h=H;
    numR=num;
    spaceX=w/numR;
    spaceY=h/numR;
    grid=new Block[numR*numR];
    int cnt=0;
    for(int i=0; i<w; i+=spaceX){
      for(int j=0; j<h; j+=spaceY){
        grid[cnt++]=new Block(j,i,spaceX,spaceY);
      }
    }
    for(int i=0; i<grid.length; i++)
      grid[i].show();
  }

  public void showBlocks(){
    for(int i=0; i<grid.length; i++)
      grid[i].show();
  }

  public void update(int x, int y){
    int i=0;
    while(!grid[i++].contains(x,y) && i<grid.length){}
    i--;
    grid[i].live();
    grid[i].show();
    println(grid[i]);
  }
  
  public Block[] getGrid(){
    return grid;
  }
  
  public void clear(){
    for(int i=0; i<grid.length; i++){
      grid[i].clear();
    }
    showBlocks();
  }
  
  public int getLongestRow(){
    int row=0;
    int longest=-1;
    int accum=0;
    int oldAccum=0;
    while(row<numR){
      accum=0;
      for(int col=0; col<numR; col++){
        if(grid[row*numR+col].isLit()){
          accum=col;
        }
      }
      if(accum>oldAccum && row>longest){
        longest=row;
        oldAccum=accum;
      }
      row++;
    }

    return accum;
  }
  
  public Phrase[] parseGrid(){
    Phrase[] phr=new Phrase[numR];
    for(int row=0; row<numR; row++){
      phr[row]=new Phrase();
      for(int col=0; col<numR; col++){
        if(grid[row*numR+col].isLit())
          phr[row].addNote(new Note(row+30,.25f,100));
        else{
          phr[row].addRest(new Rest(0.25f));
        }
      }
    }
    return phr;
  }
}
public class Muse implements JMC{
  Score score;
  
  public Muse(){
    score=new Score();
    score.setTempo(120);
    Play.midi(new Note(60,1),false);
  }
  public void play(){
    Play.midi(score,false);
    score.empty();
    println("--Emptying score--");
  }
  public void setScore(){
    score.clean();
  }
  public void stop(){
    Play.stopMidi();
  }
}

  static public void main(String args[]) {
    PApplet.main(new String[] { "--bgcolor=#ECE9D8", "GridMusic" });
  }
}
